function [P,Q] = igrf_pq(t)
% IGRF_PQ Calculates the P and Q legendre polynomials for the IGRF calculations
%   [P,Q] = IGRF_PQ(T) Calculates the Schmidt quasi-normalized associated
%   Legendre functions of degree 1-13 and order 0-13, for the co-latitudes T. T
%   can be scalar, vector, or matrix. P and Q will both be of size <104xN>,
%   where N = numel(T), where P is the Legendre functions P(cos(T)), and Q is
%   the first derivative of P: Q = dP/dT.
%
%   The n-m order of the resulting P and Q matrices is the same as the order of
%   the coefficients from IGRF_GH; namely:
%
%       n | 1  1  2  2  2  3  3  3  3 ...
%       m | 0  1  0  1  2  0  1  2  3 ...
%
%------------------------------------------------------------------------------%
%
%   Version 1.0  Date: 2016/02/05
%
%   Copyright 2016 Aaron Hendry <aaron@hendry.co.nz>
%   Otago University Space Physics Department, Dunedin, New Zealand
%       <http://www.physics.otago.ac.nz/space>
%   Released under the Simplified BSD License:
%       <http://opensource.org/licenses/BSD-2-Clause>
%
%------------------------------------------------------------------------------%

t = t(:);

ct = cos(t);
st = sin(t);

% The normalisation values are slightly expensive to calculate, so we
% pre-calculate them to avoid having to do multiple calculations of them.
persistent norm ind

if isempty(norm)
    ind = [15,2,28,16,3,40,29,17,4,51,41,30,18,5,61,52,42,31,19,6,70,62,53,...
        43,32,20,7,78,71,63,54,44,33,21,8,85,79,72,64,55,45,34,22,9,91,86,...
        80,73,65,56,46,35,23,10,96,92,87,81,74,66,57,47,36,24,11,100,97,93,...
        88,82,75,67,58,48,37,25,12,103,101,98,94,89,83,76,68,59,49,38,26,13,...
        105,104,102,99,95,90,84,77,69,60,50,39,27,14];
    n = (1:14)';
    n = n(:,ones(1,14));
    m = flipud(tril(flipud(n)));
    n = tril(n);
    m=m(~~m)-1;
    n=n(~~n)-1;

    norm = sqrt(2*factorial(n-m)./factorial(n+m))'.*(-1).^m';
    norm(m==0) = 1;
end

n=(1:13);
n21 = (1:2:25);
n21c = bsxfun(@times,n21,ct);
n21s = bsxfun(@times,n21,st);

% P can be calculated recursively - MATLAB is not good at recursion, so I have
% done the recursion manually.
P = ones(numel(t),105);
P(:,2:14) = cumprod(bsxfun(@times,-n21,st),2);
P(:,15:27) = n21c.*P(:,1:13);
P(:,28:39) = 0.5*(n21c(:,2:end).*P(:,15:26) - bsxfun(@times,(1:2:23),P(:,1:12)));
P(:,40:50) = (n21c(:,3:end).*P(:,28:38) - bsxfun(@times,(2:2:22),P(:,15:25)))/3;
P(:,51:60) = 0.25*(n21c(:,4:end).*P(:,40:49) - bsxfun(@times,(3:2:21),P(:,28:37)));
P(:,61:69) = 0.2*(n21c(:,5:end).*P(:,51:59) - bsxfun(@times,(4:2:20),P(:,40:48)));
P(:,70:77) = (n21c(:,6:end).*P(:,61:68) - bsxfun(@times,(5:2:19),P(:,51:58)))/6;
P(:,78:84) = (n21c(:,7:end).*P(:,70:76) - bsxfun(@times,(6:2:18),P(:,61:67)))/7;
P(:,85:90) = 0.125*(n21c(:,8:end).*P(:,78:83) - bsxfun(@times,(7:2:17),P(:,70:75)));
P(:,91:95) = (n21c(:,9:end).*P(:,85:89) - bsxfun(@times,(8:2:16),P(:,78:82)))/9;
P(:,96:99) = 0.1*(n21c(:,10:end).*P(:,91:94) - bsxfun(@times,(9:2:15),P(:,85:88)));
P(:,100:102) = (n21c(:,11:end).*P(:,96:98) - bsxfun(@times,(10:2:14),P(:,91:93)))/11;
P(:,103:104) = (n21c(:,12:end).*P(:,100:101) - bsxfun(@times,(11:2:13),P(:,96:97)))/12;
P(:,105) = (25.*P(103).*ct - 12.*P(100))/13;

% Calculating Q, similar to P.
Q = zeros(numel(t),105);
Q(:,2:14) = -bsxfun(@times,n,P(:,15:27));
Q(:,15:27) = (n21c.*Q(:,1:13) - n21s.*P(:,1:13));
Q(:,28:39) = 0.5*(n21c(:,2:end).*Q(:,15:26) - n21s(:,2:end).*P(:,15:26) - bsxfun(@times,(1:2:23),Q(:,1:12)));
Q(:,40:50) = (n21c(:,3:end).*Q(:,28:38) - n21s(:,3:end).*P(:,28:38) - bsxfun(@times,(2:2:22),Q(:,15:25)))/3;
Q(:,51:60) = 0.25*(n21c(:,4:end).*Q(:,40:49) - n21s(:,4:end).*P(:,40:49) - bsxfun(@times,(3:2:21),Q(:,28:37)));
Q(:,61:69) = 0.2*(n21c(:,5:end).*Q(:,51:59) - n21s(:,5:end).*P(:,51:59) - bsxfun(@times,(4:2:20),Q(:,40:48)));
Q(:,70:77) = (n21c(:,6:end).*Q(:,61:68) - n21s(:,6:end).*P(:,61:68) - bsxfun(@times,(5:2:19),Q(:,51:58)))/6;
Q(:,78:84) = (n21c(:,7:end).*Q(:,70:76) - n21s(:,7:end).*P(:,70:76) - bsxfun(@times,(6:2:18),Q(:,61:67)))/7;
Q(:,85:90) = 0.125*(n21c(:,8:end).*Q(:,78:83) - n21s(:,8:end).*P(:,78:83) - bsxfun(@times,(7:2:17),Q(:,70:75)));
Q(:,91:95) = (n21c(:,9:end).*Q(:,85:89) - n21s(:,9:end).*P(:,85:89) - bsxfun(@times,(8:2:16),Q(:,78:82)))/9;
Q(:,96:99) = 0.1*(n21c(:,10:end).*Q(:,91:94) - n21s(:,10:end).*P(:,91:94) - bsxfun(@times,(9:2:15),Q(:,85:88)));
Q(:,100:102) = (n21c(:,11:end).*Q(:,96:98) - n21s(:,11:end).*P(:,96:98) - bsxfun(@times,(10:2:14),Q(:,91:93)))/11;
Q(:,103:104) = (n21c(:,12:end).*Q(:,100:101) - n21s(:,12:end).*P(:,100:101) - bsxfun(@times,(11:2:13),Q(:,96:97)))/12;
Q(:,105) = (25.*Q(103).*ct - 12.*Q(100))/13;

% Normalise P by the Schmidt normalisation procedure.
P = bsxfun(@times,P,norm)';
Q = bsxfun(@times,Q,norm)';
% Reorder P and Q to be in the right order
P = P(ind,:);
Q = Q(ind,:);
