function [r,mlat,mlon] = trace_fieldline(rh,lat,lon,time,maxiter)
% TRACE_FIELDLINE Traces an IGRF field line to the dipole magnetic equator
%   [R,MLAT,MLON] = TRACE_FIELDLINE(RH,LAT,LON,TIME) Traces an IGRF field line
%   from the specified start point to the dipole geomagnetic equator. RH gives
%   the starting altitude (in RE), while LAT and LON give the starting
%   geographic latitude and longitude (in degrees). RH, LAT, and LON can be
%   scalars or COLUMN vectors (not row vectors), but must be the same size. TIME
%   is a MATLAB DATENUM variable specifying the epoch to calculate the IGRF
%   field line for.
%
%   TRACE_FIELDLINE(...,maxiter) As above, but forces the code to stop after
%   MAXITER iterations. By default this value is set to 1000, which should allow
%   almost all field lines to be traced to completion. Setting this to values
%   lower than ~100 should speed up code execution, as the expense of the
%   accuracy of the final result.
%
%   This code is based on the GEOCOR subroutine in the GEO-CGM FORTRAN code.
%
%------------------------------------------------------------------------------%
%
%   Version 1.0  Date: 2016/02/05
%
%   Copyright 2016 Aaron Hendry <aaron@hendry.co.nz>
%   Otago University Space Physics Department, Dunedin, New Zealand
%       <http://www.physics.otago.ac.nz/space>
%   Released under the Simplified BSD License:
%       <http://opensource.org/licenses/BSD-2-Clause>
%
%------------------------------------------------------------------------------%

if nargin==4
    maxiter = 1000;
end

lat = pi/180*lat; % Convert to radians
lon = pi/180*lon;

% tolerance for stopping point
err = 0.0001571;

% Convert to spherical magnetic units
[x,y,z] = sph2cart(rh,pi/2-lat,lon);
% Convert to MAG coordinates
[xm,ym,zm] = geo2mag(x,y,z,time);
% Convert back to spherical units
ltm = atan2(sqrt(xm.^2+ym.^2),zm);

% Distance to the (dipole) magnetic equator
rl = rh./sin(ltm).^2;
zm_start = zm; % tells us the starting equator

frac = 0.03./(1 + 3./(rl - 0.6));
frac(zm_start < 0) = -frac(zm_start < 0);

ds = rh.*frac;

x1 = x;
y1 = y;
z1 = z;

i=1;
filt = true(size(x));

while any(filt(:))
    
    [x(filt),y(filt),z(filt)] = step(x(filt),y(filt),z(filt),ds(filt),time);;
    [xm,ym,zm] = geo2mag(x,y,z,time);
    rm = xm.^2+ym.^2;
    ltm = atan2(sqrt(rm),zm);
    rm = sqrt(rm+zm.^2);

    % If we've traced too long - we're probably not going to find a result
    if i >= maxiter
        break;
    end
    
    filt_rm = isnan(x) | rm < rh;
    x(filt_rm) = nan;
    y(filt_rm) = nan;
    z(filt_rm) = nan;
    filt = ~filt_rm & ~(abs(pi/2-ltm)<err);
    filt_retry = filt & sign(zm) ~= sign(zm_start);
    filt_cont = filt & sign(zm) == sign(zm_start);
    x(filt_retry) = x1(filt_retry);
    y(filt_retry) = y1(filt_retry);
    z(filt_retry) = z1(filt_retry);
    ds(filt_retry) = ds(filt_retry)/2;
    x1(filt_cont) = x(filt_cont);
    y1(filt_cont) = y(filt_cont);
    z1(filt_cont) = z(filt_cont);
    ds(filt_cont) = rm(filt_cont).*frac(filt_cont);
    i = i+1;
end

[xm,ym,zm] = geo2mag(x,y,z,time);
[r,lat,lon] = cart2sph(xm,ym,zm);

st = sin(lat);
lat = pi/2 - atan(st.*sqrt(abs(rh./(r-rh.*st.*st))));
lat(zm_start < 0) = -lat(zm_start < 0);
sn = sin(pi/2-lat);

r = rh./(sn.*sn);
% Convert back to degrees
mlat = lat*180/pi;
mlon = lon*180/pi;